/* Settings
 *
 * Authors: Florian Boor <fb@kernelconcepts.de>
 *          Gang Wang <gang.wang@orange-ftgroup.com>
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#ifndef	_SETTINGS_H
#define	_SETTINGS_H

#define	FALSE	(0)
#define	TRUE	(!FALSE)

typedef int   boolean;

typedef enum
{
  SETTING_ERROR_SUCCESS = 0, /*success*/
  SETTING_ERROR_BAD_KEY = 1, /*The key name is invalid, maybe it includes some invalid characters*/
  SETTING_ERROR_PARSE_ERROR = 2, /*Syntax error while parsing*/ 
  SETTING_ERROR_NO_PERM = 3, /*No permission for the performance*/
  SETTING_ERROR_TYPE_MISMATCH = 4, /*type requested doesn't match type found*/ 
  SETTING_ERROR_ENTRY_EXIST = 5, /*the entry already exists*/
  SETTING_ERROR_ENTRY_NOT_EXIST = 6, /*the entry doesn't exist*/ 
  SETTING_ERROR_BACKEND_LOST = 7 /*connectioin to the setting backend is lost*/
} error_t;

typedef enum  
{
  SETTING_TYPE_INT = 0,
  SETTING_TYPE_BOOLEAN,
  SETTING_TYPE_STRING,
  SETTING_TYPE_INVALID
} type_t;

typedef enum 
{
  SETTING_ACTION_REMOVE = 0, /*the entry has just been removed*/
  SETTING_ACTION_ADD, /*a new entry has just been added*/
  SETTING_ACTION_CHANGE /*the value of the entry has just been changed*/
} action_t;

typedef struct 
{
    char * key; 
    type_t type;
    union 
    {
       int int_data;
       char * string_data;
       boolean bool_data;
    } value;
     
} entry_t;

/**
 * SettingNotifyFunc: 
 * @setting: the handle to the setting backend
 * @id: returned from setting_notify_add.
 * @key: the key of an entry.
 * @type: the type of the value stored in the entry 
 * @action: an action_t, telling what had happend to the entry.
 * @user_data: user provided data from setting_notify_add.
 * 
 * The prototype of callback function for handling notifications.
 * 
 * 
 **/
typedef void (*SettingNotifyFunc) (int setting, int id, const char *key, type_t type, action_t action, void * user_data);

int setting_open (void);

void setting_close (int setting);

error_t setting_notify_add (int setting, const char *key, SettingNotifyFunc func, void *user_data, int *id);

void setting_notify_remove (int setting, int id);

error_t setting_set_int (int setting, const char *key, int value);

error_t setting_set_boolean (int setting, const char *key, boolean value);

error_t setting_set_string (int setting, const char *key, const char * value);

error_t setting_get_int (int setting, const char * key, int *value);

error_t setting_get_boolean (int setting, const char * key, boolean *value);

error_t setting_get_string (int setting, const char * key, char ** value);

boolean setting_entry_exist (int setting, const char * key);

error_t setting_entry_get_type (int setting, const char *key, type_t * type);

#endif  /* _SETTINGS_H */
